#ifndef _LASSOCAPI_H_
#define _LASSOCAPI_H_

/*
///////////////////////////////////
// TO DO
//	clean up osConfig.h
//	clean up osDebug.h (for error codes, maybe rename the file)
//  make sure encoding methods are defined somewhere
//	parent tag stuff
//	tag callbacks to manip. lists
//      security calls
///////////////////////////////////
*/

#include "osConfig.h"
#include "osErrors.h"
#include "RequestParams.h"
#include "LassoDefines.h"


#ifdef __cplusplus
extern "C"
{
    using namespace LPTypes;

#endif

#ifdef WIN32
#ifdef EXPORT_LCAPI /* external modules don't define anything */
#define LCAPICALL __declspec(dllexport)
#else
#define LCAPICALL __declspec(dllimport)
#endif
#else
#ifdef EXPORT_LCAPI /* external modules don't define anything */
#define LCAPICALL // automatically all exported
#else
#define LCAPICALL extern
#endif
#endif

#define LCAPI_VERSION_1 1 // Lasso Version 5. First LCAPI release.
#define LCAPI_VERSION_2 2 // Lasso Version 6. Second LCAPI release
#define LCAPI_VERSION 3 // Lasso Version 7. Third LCAPI release

/*********************************************************
	LASSO VALUES
**********************************************************/
/* 
	used for shuffling data into and out of Lasso

	for any function that returns one of these Lasso will 
	garbage the struct when the module call is complete

	in cases where the data would not be named (for instance, HTTP header information)
	just the data member will be set
	
	also, for tag parameters that do not contain a "value", just the data member will be set
*/

typedef struct 
{
	const char * name;
	unsigned int nameSize;

	const char * data;
	unsigned int dataSize;
	
	LP_TypeDesc type;
} lasso_value_t;

typedef struct 
{
	const UChar * name;
	unsigned int nameSize;

	const UChar * data;
	unsigned int dataSize;
	
	LP_TypeDesc type;
} lasso_value_w_t;

#define INITVAL(X) { (X)->name = (X)->data = NULL; (X)->nameSize = (X)->dataSize = 0; }

/*
	special type so programmers know when Lasso
	will automatically dispose of the value
*/

typedef lasso_value_t auto_lasso_value_t;
typedef lasso_value_w_t auto_lasso_value_w_t;

/*
	allocates a lasso_value_t with the indicated data
	anything allocated with this function will NOT be garbage collected by Lasso
	and must be freed using lasso_freeValue

	for NULL terminated strings, passing in the result of strlen is ok here
*/
LCAPICALL osError lasso_allocValue( lasso_value_t * result, const char * name, unsigned int nameSize, const char * data, unsigned int dataSize, LP_TypeDesc type );
LCAPICALL osError lasso_allocValueW( lasso_value_w_t * result, const UChar * name, unsigned int nameSize, const UChar * data, unsigned int dataSize, LP_TypeDesc type );
LCAPICALL osError lasso_allocValueConv( lasso_value_t * result, const UChar * name, unsigned int nameSize, const char * nameEncoding, 
																const UChar * data, unsigned int dataSize, const char * dataEncoding, LP_TypeDesc type );
/*
	do not pass an auto_lasso_value_t to this function
*/
LCAPICALL osError lasso_freeValue( lasso_value_t * result );
LCAPICALL osError lasso_freeValueW( lasso_value_w_t * result );


/*********************************************************
	DATASOURCE ACTIONS
**********************************************************/
/*
	types of actions datasources could be called for
*/
typedef enum
{
	datasourceInit,
	datasourceTerm,
	datasourceExists,		/* does the specified datasource exist? */
					/* return osErrNoErr to indicate that it does */
					/* any other return valud means that it doesn't exist */

	datasourceNames,		/* the names of all the databases */
	datasourceTableNames,   /* the names of the tables in the specified database */

	datasourceSearch,
	datasourceAdd,
	datasourceUpdate,
	datasourceDelete,
	datasourceInfo,

	datasourceExecSQL,		/* only needs to be supported by SQL based databases */

	datasourceRandom,

	datasourceSchemaNames, /* LCAPI 6.0.2 */

	datasourceCloseConnection, /* LCAPI 7.0 */
	datasourceTickle, /* LCAPI 7.0 8 */ /*	Sent to the datasource when a -database and -table are specified, but no action, or a -nothing action.
											The database can, perhaps, set or reset its connection to the database.
											Or, it can do nothing.		*/
	datasourceDuplicate,	// FileMaker module ONLY impliments this message
	datasourceScripts,		// FileMaker module ONLY impliments this message
	datasourceImage,		// FileMaker module ONLY impliments this message

	datasourceFindAll

} datasource_action_t;

/*********************************************************
	TAG ACTIONS
**********************************************************/
/*
	types of actions tag modules could be called for
	ignore this for now. it may be put into use in a future version
*/
typedef int tag_action_t;

const tag_action_t tagFormatSubstitution = 1;

/*********************************************************
	SESSION OPTIONS
**********************************************************/
/*
	various options that can be set for the current session
*/
typedef enum
{
	optNone			/*  no options yet. this is for future use */
		
} session_option_t;
/*********************************************************
	LASSO REQUEST TOKEN/DATASOURCE TOKEN
**********************************************************/
/*
	special value passed to modules that identify the request
	the same value should be used when calling into Lasso
*/
typedef void * lasso_request_t;

/*********************************************************
	LASSO TYPE TOKEN - LCAPI 6
**********************************************************/

typedef void * lasso_type_t;

/*********************************************************
	DATASOURCE CONNECTION TOKEN - LCAPI 7
**********************************************************/

typedef void * lasso_dsconnection_t;

/*********************************************************
	MODULE SERVICE FUNCTIONS
**********************************************************/
/*
	service function for Lasso DataSources
*/
typedef osError (*lasso_ds_func)(lasso_request_t token, datasource_action_t action, const auto_lasso_value_t * param);
/*
	service function for Lasso tags
*/
typedef osError (*lasso_tag_func)(lasso_request_t token, tag_action_t action);

/*********************************************************
	MODULE REGISTRATION
**********************************************************/
/*
	All Lasso modules must export a routine named "registerLassoModule".
	Modules then call back into Lasso using the following two routines.

	flags for tag modules
	OR them together for different combinations
*/
typedef void (*register_module_func)(void);

const int flag_typeInitializer =		0x00000001;
const int flag_typeSubstitutionTag =	0x00000002;
const int flag_typeAsync =				0x00000004;
const int flag_typeContainerTag =		0x00000008;
const int flag_typeInterstitial =		0x00000010; // not useful for LCAPI tag types
const int flag_typeSkipSecurityCheck =	0x00000040;
const int flag_INTERNALONLY =			0x00000080; // for future use
const int flag_typeLoopingTag =			0x00000100;
const int flag_typeEval =				0x00000200; // not useful for LCAPI tag types

LCAPICALL osError lasso_registerTagModule( const char * moduleName, const char * tagName, lasso_tag_func func, int flags, const char * description );
LCAPICALL osError lasso_registerDSModule( const char * moduleName, lasso_ds_func func, int flags );

LCAPICALL osError lasso_registerTagModuleW( const UChar * moduleName, const UChar * tagName, lasso_tag_func func, int flags, const UChar * description );
LCAPICALL osError lasso_registerDSModuleW( const UChar * moduleName, lasso_ds_func func, int flags );


/*********************************************************
	REQUEST PARAMS
**********************************************************/
/*
	fetches an internal variable
*/
LCAPICALL osError lasso_getRequestParam( lasso_request_t token, RequestParamKeyword key, auto_lasso_value_t * result );
LCAPICALL osError lasso_getLassoParam( lasso_request_t token, LassoParamKeyword key, auto_lasso_value_t * result );
/*
	return osErrNoErr if Lasso IS LassoLite
*/
LCAPICALL osError lasso_isLassoLite( lasso_request_t token );

/*********************************************************
	ERROR/RESULT CODES
**********************************************************/
/*
	sets the result code and message that can be displayed
*/
LCAPICALL osError lasso_setResultCode( lasso_request_t token, osError err );
LCAPICALL osError lasso_setResultMessage( lasso_request_t token, const char * msg );
LCAPICALL osError lasso_setResultMessageW( lasso_request_t token, const UChar * msg );

/*********************************************************
	LASSO TYPE FUNCTIONS - LCAPI 6

	LCAPI 6 gives programmers the ability to access 
	Lasso's internal type system from within their LCAPI 
	tags.
**********************************************************/

/* 
 * ALLOCATING AND FREEING TYPE INSTANCES 
 */
// lasso_typeAlloc will attempt to allocate a new type instance. The name of the type is signified by the second parameter
// An array of lasso_type_t parameters can be passed to the type initializer
LCAPICALL osError lasso_typeAlloc ( lasso_request_t token, const char * typeName, int paramCount, lasso_type_t * paramsArray, lasso_type_t * outType );
LCAPICALL osError lasso_typeAllocW ( lasso_request_t token, const UChar * typeName, int paramCount, lasso_type_t * paramsArray, lasso_type_t * outType );

// lasso_typeAllocCustom is used within lasso_tag_funcs that were registered as being a type initializer (flag_typeInitializer)
// it initializes a blank custom type and sets the type's __type_name__ member to the provided value
// the new type does not yet have a lineage and has no members added to it besides __type_name__
// new data or tag members should be added using lasso_typeAddMember
// any inherited members will be added to the type after the LCAPI call returns.
// WARNING NOTE NOTE NOTE WARNING
// Do NOT call this unless you are in a type initializer. If you are not in a type initializer, the result will be a type that
// will never be fully initialized
// WARNING NOTE NOTE NOTE WARNING
LCAPICALL osError lasso_typeAllocCustom( lasso_request_t token, lasso_type_t * outCustom, const char * name );
LCAPICALL osError lasso_typeAllocCustomW( lasso_request_t token, lasso_type_t * outCustom, const UChar * name );

// Alloc a new type based on the given type. The given type's tag members will be referenced in the new type
// No data members are added except for the typename member
// Proto must be a custom type
LCAPICALL osError lasso_typeAllocFromProto( lasso_request_t token, lasso_type_t proto, lasso_type_t * out );

// Alloc a new type with the given name. The type does not have to have been registered as a type initializer or registered *at all*
// The new type will have no tag or data members, but those may be added using the appropriate LCAPI call at any time
// If no parent type is provided (a NULL pointer  or empty string is passed in), type null will be the default
// If a parent type is provided, it must have been a validly registered type initializer
// onCreate will be called for the parent and beyond
LCAPICALL osError lasso_typeAllocOneOff( lasso_request_t token, const char * name, const char * parentTypeName, lasso_type_t * out );
LCAPICALL osError lasso_typeAllocOneOffW( lasso_request_t token, const UChar * name, const UChar * parentTypeName, lasso_type_t * out );

// lasso_typeFree will attempt to free a type created through lasso_typeAlloc or any other method
// the lasso_request_t token may be NULL if the provided type has been "detached" using lasso_typeDetach
LCAPICALL osError lasso_typeFree ( lasso_request_t token, lasso_type_t inType);

// The following functions allocate instances of specific native/internal types
// The lasso_request_t token may be null. If it is null, the allocated types are created as "detached" and must be manually freed using lasso_typeFree
LCAPICALL osError lasso_typeAllocNull ( lasso_request_t token, lasso_type_t * outNull );
LCAPICALL osError lasso_typeAllocString ( lasso_request_t token, lasso_type_t * outString, const char * value, int length );
LCAPICALL osError lasso_typeAllocStringConv ( lasso_request_t token, lasso_type_t * outString, const char * value, int length, const char * conv );
LCAPICALL osError lasso_typeAllocStringW ( lasso_request_t token, lasso_type_t * outString, const UChar * value, int length );
LCAPICALL osError lasso_typeAllocInteger ( lasso_request_t token, lasso_type_t * outInteger, osInt64 value );
LCAPICALL osError lasso_typeAllocDecimal ( lasso_request_t token, lasso_type_t * outDecimal, double value );
LCAPICALL osError lasso_typeAllocDecimal2 ( lasso_request_t token, lasso_type_t * outDecimal, double value, int precision );
LCAPICALL osError lasso_typeAllocPair ( lasso_request_t token, lasso_type_t * outPair, lasso_type_t first, lasso_type_t second );
LCAPICALL osError lasso_typeAllocReference( lasso_request_t token, lasso_type_t * outRef, lasso_type_t referenced );
LCAPICALL osError lasso_typeAllocTag ( lasso_request_t token, lasso_type_t * outTag, lasso_tag_func nativeTagFunction );
// make sure the source text is UTF8 BOM or it will get the default platform encoding
LCAPICALL osError lasso_typeAllocTagFromSource ( lasso_request_t token, lasso_type_t * outTag, const char * source, int length );
LCAPICALL osError lasso_typeAllocArray ( lasso_request_t token, lasso_type_t * outArray, int count, lasso_type_t * elements );
LCAPICALL osError lasso_typeAllocMap ( lasso_request_t token, lasso_type_t * outMap, int count, lasso_type_t * elements );
LCAPICALL osError lasso_typeAllocBoolean( lasso_request_t token, lasso_type_t * outBool, bool inValue );

/*
 * Getting... Setting...
 */
// The following functions get and set the data on a previously created type instance.
// When setting a value, the type is converted if required.
// The lasso_request_t token may be null.
LCAPICALL osError lasso_typeGetString( lasso_request_t token, lasso_type_t type, auto_lasso_value_t * val );
// Depending on the specified converter, the resulting type from the following call may not always actually be of type 'string'
LCAPICALL osError lasso_typeGetStringConv( lasso_request_t token, lasso_type_t type, auto_lasso_value_t * val, const char * conv );
LCAPICALL osError lasso_typeGetStringW( lasso_request_t token, lasso_type_t type, auto_lasso_value_w_t * val );
LCAPICALL osError lasso_typeGetInteger( lasso_request_t token, lasso_type_t type, osInt64 * out );
LCAPICALL osError lasso_typeGetDecimal( lasso_request_t token, lasso_type_t type, double * out );
LCAPICALL osError lasso_typeGetBoolean( lasso_request_t token, lasso_type_t type, bool * out );

LCAPICALL osError lasso_typeSetString( lasso_request_t token, lasso_type_t type, const char * val, int len );
// Depending on the specified converter, the resulting type from the following call may not always actually be of type 'string'
LCAPICALL osError lasso_typeSetStringConv( lasso_request_t token, lasso_type_t type, const char * val, int len, const char * conv );
LCAPICALL osError lasso_typeAppendString( lasso_request_t token, lasso_type_t type, const char * val, int len );
LCAPICALL osError lasso_typeSetStringW( lasso_request_t token, lasso_type_t type, const UChar * val, int len );
LCAPICALL osError lasso_typeAppendStringW( lasso_request_t token, lasso_type_t type, const UChar * val, int len );

LCAPICALL osError lasso_typeSetInteger( lasso_request_t token, lasso_type_t type, osInt64 val );
LCAPICALL osError lasso_typeSetDecimal( lasso_request_t token, lasso_type_t type, double val );
LCAPICALL osError lasso_typeSetDecimal2( lasso_request_t token, lasso_type_t type, double val, int precision );
LCAPICALL osError lasso_typeSetBoolean( lasso_request_t token, lasso_type_t type, bool val );

// The following functions are used to manipulate the more complex native/internal types
// The lasso_request_t token may be null.
// ARRAY
LCAPICALL osError lasso_arrayGetSize( lasso_request_t token, lasso_type_t array, int * len );
LCAPICALL osError lasso_arrayGetElement( lasso_request_t token, lasso_type_t array, int index, lasso_type_t * out );
LCAPICALL osError lasso_arraySetElement(lasso_request_t token, lasso_type_t array, int index, lasso_type_t elem );
LCAPICALL osError lasso_arrayRemoveElement( lasso_request_t token, lasso_type_t array, int index );
// MAP
LCAPICALL osError lasso_mapGetSize( lasso_request_t token, lasso_type_t mp, int * len );
LCAPICALL osError lasso_mapFindElement( lasso_request_t token, lasso_type_t mp, lasso_type_t key, lasso_type_t * out );

// If the lasso_request_t token is null, the result of lasso_mapGetNthElement, outPair,  must be manually freed
LCAPICALL osError lasso_mapGetNthElement( lasso_request_t token, lasso_type_t mp, int index, lasso_type_t * outPair );

LCAPICALL osError lasso_mapSetElement( lasso_request_t token, lasso_type_t mp, lasso_type_t key, lasso_type_t value );
LCAPICALL osError lasso_mapRemoveElement( lasso_request_t token, lasso_type_t mp, lasso_type_t key );
// PAIR
LCAPICALL osError lasso_pairGetFirst( lasso_request_t token, lasso_type_t pr, lasso_type_t * out );
LCAPICALL osError lasso_pairGetSecond( lasso_request_t token, lasso_type_t pr, lasso_type_t * out );
LCAPICALL osError lasso_pairSetFirst( lasso_request_t token, lasso_type_t pr, lasso_type_t frst );
LCAPICALL osError lasso_pairSetSecond( lasso_request_t token, lasso_type_t pr, lasso_type_t scnd );

// Custom types can have a pointer value associated with them.
// The value can be used for whatever purpose the programmer needs.
//LCAPICALL osError lasso_typeSetCustomPtr( lasso_type_t the_custom_type, void * the_ptr );
//LCAPICALL osError lasso_typeGetCustomPtr( lasso_type_t the_custom_type, void ** the_ptr );

/*
 * Twiddle...
 */

// The basis of table-level automatic encoding and translation
// Converts a Lasso type INTO single-byte or binary data using the specific encoding name
// The default for all database, column, table names should be "iso8859-1"
LCAPICALL osError lasso_handleExternalConversion(lasso_request_t token, lasso_type_t instance, const char * encoding, auto_lasso_value_t * outVal);
// Converts a single-byte or binary representation of a Lasso type BACK INTO an instance of that type.
LCAPICALL osError lasso_handleInternalConversion(lasso_request_t token, const char * src, unsigned int srcLen, const char * encoding, LP_TypeDesc closestLassoType, lasso_type_t * outType);

// Change the inheritence structure of a type
// Sets newParent to be the new parent of child
// Any parent that child currently has will be destroyed
LCAPICALL osError lasso_typeInheritFrom( lasso_request_t token, lasso_type_t child, lasso_type_t newParent );

LCAPICALL osError lasso_typeGetParent( lasso_request_t token, lasso_type_t from, lasso_type_t * outParent );
LCAPICALL osError lasso_typeGetChild( lasso_request_t token, lasso_type_t from, lasso_type_t * outChild );

// Adds a function to be called after onDestroy
LCAPICALL osError lasso_typeSetCustomDtor( lasso_type_t the_custom_type, void (*dtor)(lasso_type_t the_custom_type) );

// lasso_typeAddMember is used to add members to type instances.
LCAPICALL osError lasso_typeAddMember( lasso_request_t token, lasso_type_t to, const char * named, lasso_type_t member );
LCAPICALL osError lasso_typeAddMemberW( lasso_request_t token, lasso_type_t to, const UChar * named, lasso_type_t member );

// lasso_typeGetMember is used to retrieve a member from a type instance. 
// Members are searched for by name with tag members being searched first.
// Data members are searched for if no tag is found with the given name.
LCAPICALL osError lasso_typeGetMember( lasso_request_t token, lasso_type_t from, const char * named, lasso_type_t * out );
LCAPICALL osError lasso_typeGetMemberW( lasso_request_t token, lasso_type_t from, const UChar * named, lasso_type_t * out );

// Allows storage of an opaque value
LCAPICALL osError lasso_setPtrMember(lasso_request_t token, lasso_type_t self, const char * name, void * data);
LCAPICALL osError lasso_getPtrMember(lasso_request_t token, lasso_type_t self, const char * name, void ** data);
LCAPICALL osError lasso_setPtrMemberW(lasso_request_t token, lasso_type_t self, const UChar * name, void * data);
LCAPICALL osError lasso_getPtrMemberW(lasso_request_t token, lasso_type_t self, const UChar * name, void ** data);

// currently o for internal usage
LCAPICALL osError lasso_typeSetFlag(lasso_request_t token, lasso_type_t type, int flag);
LCAPICALL osError lasso_typeHasFlag(lasso_request_t token, lasso_type_t type, int flag);
LCAPICALL osError lasso_typeClearFlag(lasso_request_t token, lasso_type_t type, int flag);

// lasso_typeGetProperties has two uses. If the targetType parameter is non-null, it is used to get all the data and tag members from a given type. 
// They are returned as a pair of arrays in the outPair value.
// The first element of each pair is the map of data members for the type. The second element is the map of tag members.
// Each element in the array represents the members of each type the targetType inherits from (its parent)
//
// If the targetType parameter is NULL, lasso_typeGetProperties will return an array containing the variable maps for the currently active request.
LCAPICALL osError lasso_typeGetProperties ( lasso_request_t token, lasso_type_t targetType, lasso_type_t * outPair );

// lasso_getTagSelf is used in LCAPI tags that are members of a custom type.
// It returns the type instance that the current call was a member of.
LCAPICALL osError lasso_getTagSelf ( lasso_request_t token, lasso_type_t * self );

// lasso_typeGetName Returns the name of the target type.
LCAPICALL osError lasso_typeGetName( lasso_request_t token, lasso_type_t target, auto_lasso_value_t * outName );
LCAPICALL osError lasso_typeGetNameW( lasso_request_t token, lasso_type_t target, auto_lasso_value_w_t * outName );
LCAPICALL osError lasso_typeIsA( lasso_request_t token, lasso_type_t target, LP_TypeDesc type );
LCAPICALL osError lasso_typeIsA2W( lasso_request_t token, lasso_type_t target, const UChar * typeName );
LCAPICALL osError lasso_typeIsA2( lasso_request_t token, lasso_type_t target, const char * typeName );

// lasso_returnTagValue specifies the return value of the current LCAPI call.
// Any type can be returned. 
// NOTE: Lasso will copy the provided value. To avoid this copy, return a hard reference using lasso_typeAllocReference.
LCAPICALL osError lasso_returnTagValue( lasso_request_t token, lasso_type_t value );

LCAPICALL osError lasso_returnTagValueBoolean( lasso_request_t token, bool b );
LCAPICALL osError lasso_returnTagValueInteger( lasso_request_t token, osInt64 i );
LCAPICALL osError lasso_returnTagValueString( lasso_request_t token, const char * p, int l );
LCAPICALL osError lasso_returnTagValueStringW( lasso_request_t token, const UChar * p, int l );
LCAPICALL osError lasso_returnTagValueDecimal( lasso_request_t token, double d );

// this function provides direct access to the tag's return value
// in many cases, directly manipulating this value can yield better performance than lasso_returnTagValue
LCAPICALL osError lasso_getTagReturnValue( lasso_request_t token, lasso_type_t * outValue );

// The following functions operate the same as their corresponding LCAPI 5.0 counterparts, but allow the tag's parameter to be 
// retrieved while preserving the type. LCAPI 5.0 calls converted all parameters to strings.
LCAPICALL osError lasso_getVariable2( lasso_request_t token, const char * key, lasso_type_t * value );
LCAPICALL osError lasso_getVariable2W( lasso_request_t token, const UChar * key, lasso_type_t * value );

LCAPICALL osError lasso_setVariable2( lasso_request_t token, const char * key, lasso_type_t value );
LCAPICALL osError lasso_setVariable2W( lasso_request_t token, const UChar * key, lasso_type_t value );

LCAPICALL osError lasso_getTagParam2( lasso_request_t token, int paramIndex, lasso_type_t * result );

LCAPICALL osError lasso_findTagParam2( lasso_request_t token, const char * paramName, lasso_type_t * result );
LCAPICALL osError lasso_findTagParam2W( lasso_request_t token, const UChar * paramName, lasso_type_t * result );

// lasso_childrenRun is used to execute the contents of a container tag. Tags become containers by registering with the flag_typeContainerTag flag.
LCAPICALL osError lasso_childrenRun( lasso_request_t token, lasso_type_t * result );

// lasso_typeRunTag can be used to execute a given tag.
// The tag can be run given a specific name and parameters. 
// The tag's return value can be accessed.
// If the tag is a member tag, the instance of which it is a member can be passed using the final parameter.
// The params, returnValue and optionalTarget parameters may all be NULL
LCAPICALL osError lasso_typeRunTag ( lasso_request_t token, const char * name, lasso_type_t tagType, int paramCount, lasso_type_t * params, lasso_type_t * returnValue, lasso_type_t optionalTarget );
LCAPICALL osError lasso_typeRunTagW ( lasso_request_t token, const UChar * name, lasso_type_t tagType, int paramCount, lasso_type_t * params, lasso_type_t * returnValue, lasso_type_t optionalTarget );

// Set, Get and Clear various flags belonging to a tag instance
LCAPICALL osError lasso_tagSetFlag( lasso_request_t token, lasso_type_t tagType, int flag );
LCAPICALL osError lasso_tagHasFlag( lasso_request_t token, lasso_type_t tagType, int flag );
LCAPICALL osError lasso_tagClearFlag( lasso_request_t token, lasso_type_t tagType, int flag );

// functions related to querying a tag's parameter and return value types
LCAPICALL osError lasso_tagGetReturnType(lasso_request_t token, lasso_type_t tagType, auto_lasso_value_t * result);
LCAPICALL osError lasso_tagGetReturnTypeW(lasso_request_t token, lasso_type_t tagType, auto_lasso_value_w_t * result);

LCAPICALL osError lasso_tagSetReturnType(lasso_request_t token, lasso_type_t tagType, const char * typeName);
LCAPICALL osError lasso_tagSetReturnTypeW(lasso_request_t token, lasso_type_t tagType, const UChar * typeName);

LCAPICALL osError lasso_tagGetExpectedParamCount(lasso_request_t token, lasso_type_t tagType, int * count);
LCAPICALL osError lasso_tagGetExpectedParam(lasso_request_t token, lasso_type_t tagType, int index, auto_lasso_value_t * result, bool * required);
LCAPICALL osError lasso_tagGetExpectedParamW(lasso_request_t token, lasso_type_t tagType, int index, auto_lasso_value_w_t * result, bool * required);


// lasso_typeAssign performs an assignment of ne type to another.
// The result will be the same as if the following had been executed in LDML:
// #left_hand_side = #right_hand_side
LCAPICALL osError lasso_typeAssign( lasso_request_t token, lasso_type_t left_hand_side, lasso_type_t right_hand_side );

// Prevents the type from being destroyed once the LCAPI call returns
// Types that have been detached must eventually be destroyed using lasso_typeFree (passing NULL as the request token) or a memory leak will occur.
LCAPICALL osError lasso_typeDetach( lasso_request_t token, lasso_type_t toDetach );

// Transfer the data from one type to another type
// BOTH types must be valid and pre-allocated
// After the call, victim will still be valid, but will be of type null
LCAPICALL osError lasso_typeStealValue( lasso_request_t token, lasso_type_t theif, lasso_type_t victim );

// Creates and runs a new LCAPI call on the given lasso_tag_func. 
// If there is already an active request on the current thread, the lasso_tag_func will be run within that threads context. 
// If there is no currently active request on the current thread, a new request will be created and run based off the global context.
// The tag_action_t parameter will be passed to the lasso_tag_func and can be used to signal or pass information to the function.
LCAPICALL osError lasso_runRequest( lasso_tag_func func, tag_action_t action, int unused );

LCAPICALL osError lasso_registerConstant( const char * name, lasso_type_t val );
LCAPICALL osError lasso_registerConstantW( const UChar * name, lasso_type_t val );

// works just like the include tag
LCAPICALL osError lasso_includeRaw( lasso_request_t token, const char * path, auto_lasso_value_t * out );

// Changes the given type into a hard reference
#define MAKE_REF(X) lasso_typeAllocReference( token, &X, X )

/*********************************************************
	TAG MODULE FUNCTIONS
**********************************************************/
/*
	fetches the name of the tag that triggered this call
	i.e. in the case of: [my_tag: ...]
	the resulting value would be "my_tag"
*/
LCAPICALL osError lasso_getTagName( lasso_request_t token, auto_lasso_value_t * result);
LCAPICALL osError lasso_getTagNameW( lasso_request_t token, auto_lasso_value_w_t * result);
/*
	fetches the number of parameters that were passed to the tag
*/
LCAPICALL osError lasso_getTagParamCount( lasso_request_t token, int * result );
/*
	fetches the indicated parameter that was sent to the tag
	param indexes start at zero
*/
LCAPICALL osError lasso_getTagParam( lasso_request_t token, int paramIndex, auto_lasso_value_t * result );
LCAPICALL osError lasso_getTagParamW( lasso_request_t token, int paramIndex, auto_lasso_value_w_t * result );
/*
	returns osErrNoErr if the param WAS defined
	anything else means it wasn't
*/
LCAPICALL osError lasso_tagParamIsDefined( lasso_request_t token, const char * paramName );
LCAPICALL osError lasso_tagParamIsDefinedW( lasso_request_t token, const UChar * paramName );
/*
	finds and fetches a tag param by name
	a return value of osErrNoErr means the param was found
*/
LCAPICALL osError lasso_findTagParam( lasso_request_t token, const char * paramName, auto_lasso_value_t * result );
LCAPICALL osError lasso_findTagParamW( lasso_request_t token, const UChar * paramName, auto_lasso_value_w_t * result );
/*
	fetches the encoding method indicated for this tag [DEPRECATED]
*/
//LCAPICALL osError lasso_getTagEncoding( lasso_request_t token, encodingMethod * method );
/*
	output some data onto the page
	Lasso will take care of encoding
	this can be called as many times as needed and new data will be appended
	lasso_outputTagDataF takes the same formatting flags as printf
*/
/*
	The resulting type for lasso_outputTagBytes and lasso_outputTagData2 will be a bytes type
*/
LCAPICALL osError lasso_outputTagBytes( lasso_request_t token, const char * data, int length );
LCAPICALL osError lasso_outputTagData2( lasso_request_t token, const char * data, int length );
/*
	The resulting type for these 3 will be a string
*/
LCAPICALL osError lasso_outputTagData( lasso_request_t token, const char * data );
LCAPICALL osError lasso_outputTagData2W( lasso_request_t token, const UChar * data, int length );
LCAPICALL osError lasso_outputTagDataF( lasso_request_t token, const char * format, ... );
/*
	allows access to the result (HTTP) header
*/
LCAPICALL osError lasso_getResultHeader( lasso_request_t token, auto_lasso_value_t * result );
LCAPICALL osError lasso_getResultHeaderW( lasso_request_t token, auto_lasso_value_w_t * result );
/*
	sets the result header
	any data will be validated so as to be in the proper format
*/
LCAPICALL osError lasso_setResultHeader( lasso_request_t token, const char * header );
LCAPICALL osError lasso_setResultHeaderW( lasso_request_t token, const UChar * header );
/*
	simply appends the supplied data to the header
	any data will be validated so as to be in the proper format
*/
LCAPICALL osError lasso_addResultHeader( lasso_request_t token, const char * data );
LCAPICALL osError lasso_addResultHeaderW( lasso_request_t token, const UChar * data );
/*
	retrieves the number of variables with the specified name
*/
LCAPICALL osError lasso_getVariableCount( lasso_request_t token, const char * named, int * count );
LCAPICALL osError lasso_getVariableCountW( lasso_request_t token, const UChar * named, int * count );
/*
	retrieves the specified variable value
*/
LCAPICALL osError lasso_getVariable( lasso_request_t token, const char * named, int index, auto_lasso_value_t * value );
LCAPICALL osError lasso_getVariableW( lasso_request_t token, const UChar * named, int index, auto_lasso_value_w_t * value );
/*
	sets the specified variable
*/
LCAPICALL osError lasso_setVariable( lasso_request_t token, const char * named, const char * value, int index );
LCAPICALL osError lasso_setVariableW( lasso_request_t token, const UChar * named, const UChar * value, int index );
/*
	removes the specified variable
*/
LCAPICALL osError lasso_removeVariable( lasso_request_t token, const char * named, int index );
LCAPICALL osError lasso_removeVariableW( lasso_request_t token, const UChar * named, int index );
/*
	finds a cookie value
*/
LCAPICALL osError lasso_getCookieValue( lasso_request_t token, const char * named, auto_lasso_value_t * value );
LCAPICALL osError lasso_getCookieValueW( lasso_request_t token, const UChar * named, auto_lasso_value_w_t * value );

/*********************************************************
	DATASOURCE MODULE FUNCTIONS
**********************************************************/
/*
	adds a result value
	datasource actions which require returning multiple values 
	can use this to add those values
	for example, this call can be used to add the name of a datasource
	that the module services or the names of the tables in a 
	particular datasource

	lasso_getInputColumn & lasso_findInputColumn can be used by tags to get input form params
*/
LCAPICALL osError lasso_addDataSourceResult( lasso_request_t token, const char * data );
/*
	Gets the currently specified database name
*/
LCAPICALL osError lasso_getDataSourceName( lasso_request_t token, auto_lasso_value_t * t, bool * useHostDefault, auto_lasso_value_t * usernamepassword );
/*
	Gets the currently specified table name
*/
LCAPICALL osError lasso_getTableName( lasso_request_t token, auto_lasso_value_t * t );
/*
	Gets the currently specified schema name
*/
LCAPICALL osError lasso_getSchemaName( lasso_request_t token, auto_lasso_value_t * schema);
/*
	Returns the host that maintains the current database
*/
LCAPICALL osError lasso_getDataHost( lasso_request_t token, auto_lasso_value_t * host, auto_lasso_value_t * usernamepassword );
LCAPICALL osError lasso_getDataHost2( lasso_request_t token, auto_lasso_value_t * host, auto_lasso_value_t * defaultSchema, auto_lasso_value_t * usernamepassword );
/*
	The number of rows that should be skipped in the found set
*/
LCAPICALL osError lasso_getSkipRows( lasso_request_t token, int * recs );
/*
	The maximum number of rows in the found set to return
*/
LCAPICALL osError lasso_getMaxRows( lasso_request_t token, int * recs );
/*
	Gets the currently specified record id (FileMaker specific) 
*/
LCAPICALL osError lasso_getRowID( lasso_request_t token, int * id );
/*
	Sets the currently specified record id (FileMaker specific) 
*/
LCAPICALL osError lasso_setRowID( lasso_request_t token, int id );
/*
	Gets the name and the value of the currently specified primary key column
*/
LCAPICALL osError lasso_getPrimaryKeyColumn( lasso_request_t token, auto_lasso_value_t * v );
LCAPICALL osError lasso_getPrimaryKeyColumn2( lasso_request_t token, int index, auto_lasso_value_t * v );

/*
	Gets the number of input columns for this database action
*/
LCAPICALL osError lasso_getInputColumnCount( lasso_request_t token, int * count );
/*
	Gets the number of sort columns for this database action
*/
LCAPICALL osError lasso_getSortColumnCount( lasso_request_t token, int * count );

/*
	Gets an individual input column by index
*/
LCAPICALL osError lasso_getInputColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
/*
	Gets an individual sort column by index
*/
LCAPICALL osError lasso_getSortColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
/*
	Gets an individual input column by name
*/
LCAPICALL osError lasso_findInputColumn( lasso_request_t token, const char * name, auto_lasso_value_t * value );
LCAPICALL osError lasso_findInputColumnW( lasso_request_t token, const UChar * name, auto_lasso_value_t * value );
/*
	Gets the logical operator for this database action
*/
LCAPICALL osError lasso_getLogicalOp( lasso_request_t token, LP_TypeDesc * op );

/*
	Gets the number of return columns for this action
*/
LCAPICALL osError lasso_getReturnColumnCount( lasso_request_t token, int * count );
/*
	Gets an individual return column by index
*/
LCAPICALL osError lasso_getReturnColumn( lasso_request_t token, int num, auto_lasso_value_t * v );

/*
	Adds information about a particular column. Column information should be added in the order in which the columns occur inthe database.
	Column information should be added no matter what the action. For the show action, only column information is added.
*/
LCAPICALL osError lasso_addColumnInfo( lasso_request_t token, const char * name, int nullOK, LP_TypeDesc type, LP_TypeDesc protection );
LCAPICALL osError lasso_addColumnInfo2( lasso_request_t token, const char * name, int nullOK, LP_TypeDesc type, LP_TypeDesc protection, const char ** valueList, int countValueList );

/*
	Add all the column data for the next result row
*/
LCAPICALL osError lasso_addResultRow( lasso_request_t token, const char ** columns, unsigned int * dataSizes, int numColumns );
/*
	Sets the number of rows found in the query. This will not always be the same value as the number of rows added with the lasso_addResultRow call
	as the skip recs and max recs parameters effect that number.
*/
LCAPICALL osError lasso_setNumRowsFound( lasso_request_t token, int num );

/*
	LCAPI 6
*/
LCAPICALL osError lasso_getDataSourceModuleName( lasso_request_t token, auto_lasso_value_t * val );

// *-*-*-*-*-*-*-*-*-*-*-*-*
// |   L C A P I >->7<-<   |
// *-*-*-*-*-*-*-*-*-*-*-*-*

// called to access the current datasource connection
// may recurse the ds call by sending it the datasourceOpenConnection message
LCAPICALL osError lasso_getDSConnection(lasso_request_t token, lasso_dsconnection_t * conn);

// called to set the current connection for the datasource
// may recurse to deliver the datasourceCloseConnection message if there is already a valid lasso_dbconnection_t
LCAPICALL osError lasso_setDSConnection(lasso_request_t token, lasso_dsconnection_t conn);

LCAPICALL osError lasso_setDSDefaultCharset(lasso_request_t token, const char * cs);

/*********************************************************
	SEMAPHORE FUNCTIONS
**********************************************************/

/*
	Semephores are a means by which access to shared resources can be control amongst several threads of execution.
	Each semephore must have a unique name. Always release semaphores as soon as possible.
*/
LCAPICALL osError lasso_createSem( lasso_request_t token, const char * name );
LCAPICALL osError lasso_destroySem( lasso_request_t token, const char * name );

LCAPICALL osError lasso_acquireSem( lasso_request_t token, const char * name );
LCAPICALL osError lasso_releaseSem( lasso_request_t token, const char * name );

/*********************************************************
	FORMATTING FUNCTIONS
**********************************************************/

/* 
	Formats the supplied buffer and put the resulting data in the data member of the auto_lasso_value_t.
	The buffer should consist of plain text and bracketed Lasso tags.
*/
LCAPICALL osError lasso_formatBuffer( lasso_request_t token, const char * buffer, auto_lasso_value_t * output );

/*********************************************************
	STORAGE FUNCTIONS
**********************************************************/

/*
  These calls can be used to have Lasso store arbitrary persistent data. This is an excellent way to store preferences.
 */
/* 
   Returns osErrNoErr if the data, specified by key, exists.
   The length of the stored data can be ruented in the length parameter.
   This parameter may be NULL if the length is not required.
 */
LCAPICALL osError lasso_storeHasData( lasso_request_t token, const char * key, unsigned int * length );

/*
  Adds the data to Lasso's storage.
  key is the unique identifier for the data.
 */
LCAPICALL osError lasso_storePutData( lasso_request_t token, const char * key, const void * data, unsigned int length );

/*
  Fetches that has been stored.
  key is the unique identifier for the data.
  The data will be returned in the "data" member of the auto_lasso_value_t parameter.
  The length of the resulting data is stored in the length parameter. This param may NOT be NULL.
 */
LCAPICALL osError lasso_storeGetData( lasso_request_t token, const char * key, auto_lasso_value_t * value, unsigned int * length );

/*********************************************************
	SECURITY FUNCTIONS
**********************************************************/

/*
  Returns osErrNoErr if the current user has administrator privs.
  This is useful for doing module administration that only the admin should be able to do.
*/
LCAPICALL osError lasso_isAdministrator( lasso_request_t token );
LCAPICALL osError lasso_isAnonymous( lasso_request_t token );

// pass "anonymous" with no password to switch to lowest privs
LCAPICALL osError lasso_switchToUser( lasso_request_t token, const char * username, const char * password );

typedef enum secure_operation_t
{
	kInspectFiles,
	kCreateFiles,
	kDeleteFiles,
	kCopyFiles,
	kMoveFiles,
	kReadFiles,
	kWriteFiles,
	kAccessOutsideRoot

} secure_operation_t;

/*
	Returns osErrNoErr is the given operation is permitted by Lasso security.
	The data perameter should be the full path to the file using the *internal pathname* format.
*/
LCAPICALL osError lasso_operationAllowed( lasso_request_t token, secure_operation_t op, void * data );

const int kMaxPathLen = 1024;
typedef char osPathname[1024];

// converts an internal pathname into a platform-specific pathname
LCAPICALL void lasso_getPlatformSpecificPath( const char* inInternalPath, osPathname outPlatformPath );

// converts a platform-specific pathname into an internal pathname
LCAPICALL void lasso_getInternalPath( const char* inPlatformPath, osPathname outInternalPath );

// converts a relative path into a from-the-server-root path
// if the path is already from the root it won't be changed
LCAPICALL void lasso_fullyQualifyPath( lasso_request_t token, const char * inRelativePath, osPathname outFullyQualified );

// converts a from-the-root path into a full internal path
LCAPICALL void lasso_resolvePath( lasso_request_t token, const char * inFromRootPath, osPathname outFullPath );

// does several of the above in one step
// takes a path, such as one which may have been passed to any Lasso tag, and converts it into a full platform specific path name
LCAPICALL void lasso_internalToFullPlatformPath( lasso_request_t token, const char * relativeOrFullInternalPath, osPathname fullPlatformPath );

/*********************************************************
	LOGGING FUNCTIONS
**********************************************************/

typedef enum log_level_t
{
	LOG_LEVEL_CRITICAL,
	LOG_LEVEL_WARNING,
	LOG_LEVEL_DETAIL,

	LOG_LEVEL_ALWAYS // always printed to window (only). used internally in a few places

} log_level_t;

// various destinations for logged information to go
// OR them together to set the preference if the information is to go to various destinations
const int flag_logDestinationConsole =	0x00000001;
const int flag_logDestinationFile =		0x00000002;
const int flag_logDestinationDatabase =	0x00000004;

// this changes the system-wide log destination preference
LCAPICALL osError lasso_logSetDestination( log_level_t msgLevel, int dest );

// log a message. the message goes to the prefered destination for the message level
// messages are limited to 2048 bytes in length
// messages sent to the console are limited to 512 bytes in length
// messages sent to the database are limited a little less than 2048 bytes since the 
//	total length of the sql statement used to insert the message is limited to 2048 bytes
LCAPICALL osError lasso_log( log_level_t msgLevel, const char * fmt, ... );

// This will cause Lasso's connection listening thread, which is normally the main application thread, to be split off into it's own thread
// This has no effect when running as an NT service
// The parameter is a function to call as the new "main loop"
// If some other entity has already called this and given its own mainloop function
//	the call will fail
// This can only be called at startup (before connections are accepted)
// Don't call this
typedef void (*capi_mainloop)(void);
LCAPICALL osError lasso_setMainLoop(capi_mainloop x);

#ifdef __cplusplus
}
#endif

#endif

