<?LassoScript

	// Valid Tags
	//
	// Note - These are convenience tags which check for the most common formatting for each value.
	// According to the RFCs and other standards a great many values which these tags return false
	// for will be technically valid.  The credit card tag checks the ROT-13 algorithm which is a
	// good first check for whether a credit card is valid.
	//
	// [Valid_Email]
	//
	// [Valid_URL]
	//
	// [Valid_Date]
	//
	// [Valid_CreditCard]
	//

	Define_Tag: 'Valid_Email', -Priority='Replace',
			-Required='email',
			-Optional='standarddomains',
			-Optional='domain',
			-Optional='hostname';

		protect;

			local: '_email' = (string: #email);

			#_email->trim;

			#_email = (string_findregexp:
				#_email,
				-find='[^<>@ ]+@[A-Za-z0-9\\-_]+\\.[A-Za-z0-9\\-_\\.]+');

			if: (#_email->type == 'array') && (#_email->size == 1) && (#_email->(get: 1) == #email);

				#_email = #_email->(get: 1);

				local:
					'lhs' = #_email->(split: '@')->(get: 1),
					'rhs' = #_email->(split: '@')->(get: 2);

				local:
					'thedomain' = #rhs->(split: '.')->last;

			else;
				return: false;
			/if;

			if: (local_defined: 'standarddomains') && ((array: 'com','gov','mil','net','org','int') !>> #thedomain);
				return: false;
			/if;

			if: (local_defined: 'domain');
				if: (#domain->(isa: 'array')) && (#domain !>> #thedomain);
					return: false;
				else: ! #domain->(isa: 'array');
					local: 'temp' = (string: #domain)->(split: ',');
					if: (#temp->(isa: 'array')) && (#temp !>> #thedomain);
						return: false;
					/if;
				/if;
			/if;

			if: (local_defined: 'hostname') && (#hostname != #rhs);
				return: false;
			/if;

			return: true;

			handle_error;
				log_critical: 'Valid_Email Syntax Error: ' (error_code) ': ' (error_msg);
				return: false;
			/handle_error;

		/protect;

	/Define_Tag;

	Define_Tag: 'Valid_URL', -Priority='Replace',
			-required='url',
			-optional='protocol';

		local: '_url' = #url;

		#_url->trim;

		Local: 'parse' = (string_findregexp: #_url, -find='(\\w+):(/?/?)([\\w\\.]+)(/?.*)');

		if: (#parse->size < 5);
			return: false;
		/if;

		if: (#_url != (#parse->(get:2) + ':' + #parse->(get:3) + #parse->(get:4) + #parse->(get:5)));
			return: false;
		/if;

		if: (local_defined: 'protocol');
			if: (#protocol->(isa: 'array')) && (#protocol !>> #parse->(get:2));
				return: false;
			else: ! #protocol->(isa: 'array');
				local: 'temp' = #protocol->(split: ',');
				if: (#temp->(isa: 'array')) && (#temp !>> #parse->(get:2));
					return: false;
				/if;
			/if;
		/if;

		return: true;

	/Define_Tag;

	Define_Tag: 'Valid_Date', -Priority='Replace',
			-Required='date',
			-Optional='format';

		if: (local_defined: 'format');
			local: 'parse' = (date: #date, -format=#format);
		else;
			local: 'parse' = (date: #date);
		/if;

		return: (#parse->type != 'null') &&
			((string: #parse)->size > 0) &&
			(#parse !>> '01/02/0000') &&
			(#parse !>> '01/01/0000');

	/Define_Tag;

	Define_Tag: 'Valid_CreditCard', -Priority='Replace',
			-Required='card',
			-optional='type';

		//
		// Clean the card number and check its length
		//

		local: '_card' = (string_replaceregexp: #card, -find='\\D', -replace='');

		if: (#_card->size < 13);
			return: false;
		/if;

		//
		// Check the ROT-13 checksum for the card number
		//

		local: 'rev' = #_card;
		#rev->reverse;
		local: 'checksum' = 0;
		iterate: #rev, (local: 'digit');
			if: (loop_count % 2) == 0;
					#checksum += (2 * (integer: #digit));
				if: (integer: #digit) >= 5;
					#checksum -= 9;
				/if;
			else;
				#checksum += (integer: #digit);
			/if;
		/iterate;

		if: (#checksum % 10) != 0;
			return: false;
		/if;

		//
		// If a type is specified check the first character and length of the credit card
		//

		if: (local_defined: 'type');
			if: (#type >> 'visa');
				if: ((#_card->size != 13) && (#_card->size != 16)) || (#_card->(get:1) != '4');
					return: false;
				/if;
			else: (#type >> 'mastercard');
				if: (#_card->size != 16) || (#_card->(get:1) != '5');
					return: false;
				/if;
			else: (#type >> 'discover');
				if: (#_card->size != 16) || (#_card->(get:1) != '6');
					return: false;
				/if;
			else: (#type >> 'amex');
				if: (#_card->size != 15) || (#_card->(get:1) != '3');
					return: false;
				/if;
			else: (#type >> 'diners');
				if: (#_card->size != 14) || (#_card->(get:1) != '3');
					return: false;
				/if;
			else: (#type >> 'jcb');
				if: (#_card->size != 15) && (#_card->size != 16);
					return: false;
				/if;
			/if;
		/if;

		return: true;

	/Define_Tag;

?>
